local lib = LibStub:NewLibrary("S2Kmisc-1.0", 1)

if not lib then
    return
end

lib.LOOT_ROLL_MSG_MATCH_TABLE =
{
    -- Handlers:
    --
    --   LOOT_ROLL_ALL_PASSED (item)
    --   LOOT_ROLL_CHOSE (player, item, choice)
    --   LOOT_ROLL_ROLLED (player, item, choice, roll)
    --   LOOT_ROLL_WON (player, item)
    --   LOOT_ROLL_WON_NO_SPAM (player, item, choice, roll)

    {
        -- Everyone passed on: %s
        msg = LOOT_ROLL_ALL_PASSED,
        handler = "LOOT_ROLL_ALL_PASSED",
        args = { "item" },
        values = {}
    },
    {
        -- %s has selected Disenchant for: %s
        msg = LOOT_ROLL_DISENCHANT,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "disenchant" }
    },
    {
        -- You have selected Disenchant for: %s
        msg = LOOT_ROLL_DISENCHANT_SELF,
        handler = "LOOT_ROLL_CHOSE",
        args = { "item" },
        values = { player = "$self$", self = true, choice = "disenchant" }
    },
    {
        -- %s has selected Greed for: %s
        msg = LOOT_ROLL_GREED,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "greed" }
    },
    {
        -- You have selected Greed for: %s
        msg = LOOT_ROLL_GREED_SELF,
        handler = "LOOT_ROLL_CHOSE",
        args = { "item" },
        values = { player = "$self$", self = true, choice = "greed" }
    },
    {
        -- %s has selected Need for: %s
        msg = LOOT_ROLL_NEED,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "need" }
    },
    {
        -- You have selected Need for: %s
        msg = LOOT_ROLL_NEED_SELF,
        handler = "LOOT_ROLL_CHOSE",
        args = { "item" },
        values = { player = "$self$", self = true, choice = "need" }
    },
    {
        -- %s passed on: %s
        msg = LOOT_ROLL_PASSED,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "pass" }
    },
    {
        -- %s automatically passed on: %s because he cannot loot that item.
        msg = LOOT_ROLL_PASSED_AUTO,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "pass" }
    },
    {
        -- %s automatically passed on: %s because she cannot loot that item.
        msg = LOOT_ROLL_PASSED_AUTO_FEMALE,
        handler = "LOOT_ROLL_CHOSE",
        args = { "player", "item" },
        values = { choice = "pass" }
    },
    {
        -- You passed on: %s
        msg = LOOT_ROLL_PASSED_SELF,
        handler = "LOOT_ROLL_CHOSE",
        args = { "item" },
        values = { player = "$self$", self = true, choice = "pass" }
    },
    {
        -- You automatically passed on: %s because you cannot loot that item.
        msg = LOOT_ROLL_PASSED_SELF_AUTO,
        handler = "LOOT_ROLL_CHOSE",
        args = { "item" },
        values = { player = "$self$", self = true, choice = "pass" }
    },
    {
        -- Disenchant Roll - %d for %s by %s
        msg = LOOT_ROLL_ROLLED_DE,
        handler = "LOOT_ROLL_ROLLED",
        args = { "roll", "item", "player" },
        values = { choice = "disenchant" }
    },
    {
        -- Greed Roll - %d for %s by %s
        msg = LOOT_ROLL_ROLLED_GREED,
        handler = "LOOT_ROLL_ROLLED",
        args = { "roll", "item", "player" },
        values = { choice = "greed" }
    },
    {
        -- Need Roll - %d for %s by %s
        msg = LOOT_ROLL_ROLLED_NEED,
        handler = "LOOT_ROLL_ROLLED",
        args = { "roll", "item", "player" },
        values = { choice = "need" }
    },
    {
        -- Need Roll - %d for %s by %s + Role Bonus
        msg = LOOT_ROLL_ROLLED_NEED_ROLE_BONUS,
        handler = "LOOT_ROLL_ROLLED",
        args = { "roll", "item", "player" },
        values = { choice = "need" }
    },
    {
        -- %s won: %s
        msg = LOOT_ROLL_WON,
        handler = "LOOT_ROLL_WON",
        args = { "player", "item" },
        values = {}
    },
    {
        -- %1$s won: %3$s |cff818181(Disenchant - %2$d)|r
        msg = LOOT_ROLL_WON_NO_SPAM_DE,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "player", "roll", "item" },
        values = { choice = "disenchant" }
    },
    {
        -- %1$s won: %3$s |cff818181(Greed - %2$d)|r
        msg = LOOT_ROLL_WON_NO_SPAM_GREED,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "player", "roll", "item" },
        values = { choice = "greed" }
    },
    {
        -- %1$s won: %3$s |cff818181(Need - %2$d)|r
        msg = LOOT_ROLL_WON_NO_SPAM_NEED,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "player", "roll", "item" },
        values = { choice = "need" }
    },
    {
        -- You won: %s
        msg = LOOT_ROLL_YOU_WON,
        handler = "LOOT_ROLL_WON",
        args = { "item" },
        values = { player = "$self$", self = true }
    },
    {
        -- You won: %2$s |cff818181(Disenchant - %1$d)|r
        msg = LOOT_ROLL_YOU_WON_NO_SPAM_DE,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "roll", "item" },
        values = { player = "$self$", self = true, choice = "disenchant" }
    },
    {
        -- You won: %2$s |cff818181(Greed - %1$d)|r
        msg = LOOT_ROLL_YOU_WON_NO_SPAM_GREED,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "roll", "item" },
        values = { player = "$self$", self = true, choice = "greed" }
    },
    {
        -- You won: %2$s |cff818181(Need - %1$d)|r
        msg = LOOT_ROLL_YOU_WON_NO_SPAM_NEED,
        handler = "LOOT_ROLL_WON_NO_SPAM",
        args = { "roll", "item" },
        values = { player = "$self$", self = true, choice = "need" }
    },
}

function lib:__BuildMsgMatchTable(tab)
    local n, match
    for n, match in pairs(tab) do
        match.__regexp = "^" .. (match.msg:gsub("([%%%?%*%-%+%.%^%$%(%)%[%]%|])", "%%%1")) .. "$"
        match.__args, match.__count = {}, 0
        local arg_num, arg_match = 1
        for arg_match in match.__regexp:gmatch("%%%%[%%%d%$]-[sd]") do
            arg_num = 0 + (arg_match:match("(%d+)%%%$") or arg_num)
            table.insert(match.__args, match.args[arg_num])
            arg_num = arg_num + 1
        end
        match.__regexp = match.__regexp:gsub("%%%%[%%%d%$]-[sd]", "(.+)")
        local n2, match2
        for n2, match2 in pairs(tab) do
            if match2.msg:match(match.__regexp) then
                match.__count = match.__count + 1
            end
        end
    end
    table.sort(tab, function(a, b) return a.__count < b.__count end)
end

function lib:ParseMsgMatchTable(tab, msg)
    if not tab[1].__regexp then
        self:__BuildMsgMatchTable(tab)
    end
    local n, match
    for n, match in pairs(tab) do
        local p = { msg:match(match.__regexp) }
        if p[1] then
            local args, pos, arg, value = {}
            for pos, arg in pairs(match.__args) do
                args[arg] = p[pos]
            end
            for arg, value in pairs(match.values) do
                if (arg == "player") then
                    if value == "$self$" then
                        value = UnitName("player")
                    end
                end
                args[arg] = value
            end
            return match.handler, args
        end
    end
end
